==> xl-scale-plugin-4.0.1-sources.jar: com/xebialabs/deployit/plugin/cloud/ci/EnvironmentTemplate.java <==
package com.xebialabs.deployit.plugin.cloud.ci;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import com.google.common.base.Joiner;
import com.google.common.base.Splitter;
import com.google.common.collect.Iterables;

import com.xebialabs.deployit.plugin.api.flow.Step;
import com.xebialabs.deployit.plugin.api.reflect.Type;
import com.xebialabs.deployit.plugin.api.udm.ControlTask;
import com.xebialabs.deployit.plugin.api.udm.Metadata;
import com.xebialabs.deployit.plugin.api.udm.Property;
import com.xebialabs.deployit.plugin.api.udm.base.BaseConfigurationItem;
import com.xebialabs.deployit.plugin.cloud.step.*;
import com.xebialabs.deployit.plugin.cloud.step.plan.SingleStepFilter;
import com.xebialabs.deployit.plugin.cloud.step.plan.StepOrderComparator;
import com.xebialabs.deployit.plugin.cloud.util.CiParser;
import com.xebialabs.deployit.plugin.cloud.util.InstanceDescriptorResolver;

import static com.google.common.base.Splitter.on;
import static com.google.common.collect.Collections2.filter;
import static com.google.common.collect.Iterables.*;
import static com.google.common.collect.Lists.newArrayList;
import static com.google.common.collect.Maps.newHashMap;
import static com.xebialabs.deployit.plugin.cloud.util.MapsHelper.putOrMerge;
import static java.util.Collections.sort;

@SuppressWarnings("serial,unused")
@Metadata(description = "Cloud environment template", root = Metadata.ConfigurationItemRoot.CONFIGURATION)
public class EnvironmentTemplate extends BaseConfigurationItem {

    private static InstanceDescriptorResolver descriptorResolver = new InstanceDescriptorResolver();

    private static CiParser ciParser = new CiParser();

    // CI properties

    @Property(required = false, size = Property.Size.LARGE, label = "Description of the template")
    private String description;

    @Property(required = true, label = "Host templates")
    private List<BaseHostTemplate> hostTemplates;

    @Property(required = true, size = Property.Size.LARGE, label = "Environment descriptor", description = "Freemarker template of XML which describes environment")
    private String xmlDescriptor;

    // Getters & setters

    public String getDescription() {
        return description;
    }

    public void setDescription(final String description) {
        this.description = description;
    }

    public void setHostTemplates(final List<BaseHostTemplate> hostTemplates) {
        this.hostTemplates = hostTemplates;
    }

    public void setXmlDescriptor(final String xmlDescriptor) {
        this.xmlDescriptor = xmlDescriptor;
    }

    public List<BaseHostTemplate> getHostTemplates() {
        return hostTemplates;
    }


    public String getXmlDescriptor() {
        return xmlDescriptor;
    }

    @ControlTask(label = "Instantiate environment", parameterType = "cloud.CloudEnvironmentParameters", description = "Instantiate environment and all hosts, which templates are linked to this environment template")
    public List<? extends Step> instantiate(CloudEnvironmentParameters parameters) {

        String environmentName = getLast(on("/").split(parameters.getEnvironmentId()));


        ArrayList<Step> steps = newArrayList();

        steps.add(new CheckParametersStep(parameters));

        // Registry of all added steps per template type
        final Map<Type, Collection<? extends Step>> waitStepsAdded = newHashMap();

        if (!getHostTemplates().isEmpty()) {

            int seq = 1;

            for (final BaseHostTemplate template : getHostTemplates()) {

                Collection<? extends Step> stepsToAdd = filter(
                        template.produceCreateSteps(environmentName, seq),
                        new SingleStepFilter(waitStepsAdded, template.getType())
                );

                steps.addAll(stepsToAdd);

                putOrMerge(waitStepsAdded, template.getType(), newArrayList(stepsToAdd));

                seq++;
            }

            steps.add(new RegisterInstancesStep(descriptorResolver, ciParser, parameters.getHostsPath()));
        }

        steps.add(new RegisterEnvironmentStep(descriptorResolver, ciParser, this, parameters.getEnvironmentId()));

        sort(steps, new StepOrderComparator());

        return steps;
    }

    @ControlTask(label = "Validate descriptor", description = "Validate XML descriptor of the environment template")
    public List<? extends Step> validateEnvironmentDescriptor() {
        List<Step> steps = newArrayList();

        for (BaseHostTemplate ht : getHostTemplates()) {
            steps.add(new ValidateHostDescriptorStep(ht.getXmlDescriptor(), ht));
        }

        steps.add(new ValidateEnvironmentDescriptorStep(this, getName()));

        return steps;
    }

}

==> xl-scale-plugin-5.0.0-sources.jar: com/xebialabs/deployit/plugin/cloud/ci/EnvironmentTemplate.java <==
package com.xebialabs.deployit.plugin.cloud.ci;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import com.xebialabs.deployit.plugin.api.flow.Step;
import com.xebialabs.deployit.plugin.api.reflect.Type;
import com.xebialabs.deployit.plugin.api.udm.ControlTask;
import com.xebialabs.deployit.plugin.api.udm.Metadata;
import com.xebialabs.deployit.plugin.api.udm.Property;
import com.xebialabs.deployit.plugin.api.udm.base.BaseConfigurationItem;
import com.xebialabs.deployit.plugin.cloud.step.CheckParametersStep;
import com.xebialabs.deployit.plugin.cloud.step.RegisterEnvironmentStep;
import com.xebialabs.deployit.plugin.cloud.step.RegisterInstancesStep;
import com.xebialabs.deployit.plugin.cloud.step.ValidateEnvironmentDescriptorStep;
import com.xebialabs.deployit.plugin.cloud.step.plan.SingleStepFilter;
import com.xebialabs.deployit.plugin.cloud.step.plan.StepOrderComparator;
import com.xebialabs.deployit.plugin.cloud.util.CiParser;
import com.xebialabs.deployit.plugin.cloud.util.InstanceDescriptorResolver;

import static com.google.common.base.Splitter.on;
import static com.google.common.collect.Collections2.filter;
import static com.google.common.collect.Iterables.getLast;
import static com.google.common.collect.Lists.newArrayList;
import static com.google.common.collect.Maps.newHashMap;
import static com.xebialabs.deployit.plugin.cloud.util.MapsHelper.putOrMerge;
import static java.util.Collections.sort;

@SuppressWarnings("serial")
@Metadata(description = "Cloud environment template", root = Metadata.ConfigurationItemRoot.CONFIGURATION)
public class EnvironmentTemplate extends BaseConfigurationItem {

    private static InstanceDescriptorResolver descriptorResolver = new InstanceDescriptorResolver();

    private static CiParser ciParser = new CiParser();

    // CI properties

    @Property(required = false, size = Property.Size.LARGE, label = "Description of the template")
    private String description;

    @Property(required = true, label = "Cloud templates")
    private List<InstantiatableCloudTemplate> hostTemplates;

    @Property(required = true, size = Property.Size.LARGE, label = "Environment descriptor", description = "Freemarker template of XML which describes environment")
    private String xmlDescriptor;

    // Getters & setters

    public String getDescription() {
        return description;
    }

    public void setDescription(final String description) {
        this.description = description;
    }

    public void setHostTemplates(final List<InstantiatableCloudTemplate> hostTemplates) {
        this.hostTemplates = hostTemplates;
    }

    public void setXmlDescriptor(final String xmlDescriptor) {
        this.xmlDescriptor = xmlDescriptor;
    }

    public List<InstantiatableCloudTemplate> getHostTemplates() {
        return hostTemplates;
    }


    public String getXmlDescriptor() {
        return xmlDescriptor;
    }

    @ControlTask(label = "Instantiate environment", parameterType = "cloud.CloudEnvironmentParameters", description = "Instantiate environment and all hosts, which templates are linked to this environment template")
    public List<? extends Step> instantiate(CloudEnvironmentParameters parameters) {

        String environmentName = getLast(on("/").split(parameters.getEnvironmentId()));


        ArrayList<Step> steps = newArrayList();

        steps.add(new CheckParametersStep(parameters));

        // Registry of all added steps per template type
        final Map<Type, Collection<? extends Step>> waitStepsAdded = newHashMap();

        if (!getHostTemplates().isEmpty()) {

            int seq = 1;

            for (final InstantiatableCloudTemplate template : getHostTemplates()) {

                Collection<? extends Step> stepsToAdd = filter(
                        template.produceCreateSteps(environmentName, seq),
                        new SingleStepFilter(waitStepsAdded, template.getType())
                );

                steps.addAll(stepsToAdd);

                putOrMerge(waitStepsAdded, template.getType(), newArrayList(stepsToAdd));

                seq++;
            }

            steps.add(new RegisterInstancesStep(parameters.getHostsPath()));
        }

        steps.add(new RegisterEnvironmentStep(descriptorResolver, ciParser, this, parameters.getEnvironmentId()));

        sort(steps, new StepOrderComparator());

        return steps;
    }

    @ControlTask(label = "Validate descriptor", description = "Validate XML descriptor of the environment template")
    public List<? extends Step> validateEnvironmentDescriptor() {
        List<Step> steps = newArrayList();

        for (InstantiatableCloudTemplate t : getHostTemplates()) {
            steps.addAll(t.produceValidationSteps());
        }

        steps.add(new ValidateEnvironmentDescriptorStep(this, getName()));

        return steps;
    }

}
